package sun.reflect.misc;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import sun.security.util.SecurityConstants;
import java.lang.reflect.Member;
import sun.reflect.Reflection;
import java.lang.reflect.Modifier;

public final class ReflectUtil
{
    public static final String PROXY_PACKAGE = "com.sun.proxy";
    
    private ReflectUtil() {
    }
    
    public static Class<?> forName(String className) throws ClassNotFoundException {
        checkPackageAccess(className);
        return Class.forName(className);
    }
    
    public static Object newInstance(Class<?> clazz) throws InstantiationException, IllegalAccessException {
        checkPackageAccess(clazz);
        return clazz.newInstance();
    }
    
    public static void ensureMemberAccess(Class<?> clazz, Class<?> clazz2, Object o, int mod) throws IllegalAccessException {
        if (o == null && Modifier.isProtected(mod)) {
            int n = (mod & 0xFFFFFFFB) | 0x1;
            Reflection.ensureMemberAccess(clazz, clazz2, o, n);
            try {
                Reflection.ensureMemberAccess(clazz, clazz2, o, n & 0xFFFFFFFE);
                return;
            }
            catch (IllegalAccessException ex) {
                if (isSubclassOf(clazz, clazz2)) {
                    return;
                }
                throw ex;
            }
        }
        else {
            Reflection.ensureMemberAccess(clazz, clazz2, o, mod);
            return;
        }
    }
    
    private static boolean isSubclassOf(Class<?> superclass, Class<?> clazz) {
        while (superclass != null) {
            if (superclass == clazz) {
                return true;
            }
            superclass = superclass.getSuperclass();
        }
        return false;
    }
    
    public static void conservativeCheckMemberAccess(Member member) throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager == null) {
            return;
        }
        Class<Object> declaringClass = member.getDeclaringClass();
        checkPackageAccess(declaringClass);
        if (Modifier.isPublic(member.getModifiers()) && Modifier.isPublic(declaringClass.getModifiers())) {
            return;
        }
        securityManager.checkPermission(SecurityConstants.CHECK_MEMBER_ACCESS_PERMISSION);
        return;
    }
    
    public static void checkPackageAccess(Class<?> clazz) {
        checkPackageAccess(clazz.getName());
        if (isNonPublicProxyClass(clazz)) {
            checkProxyPackageAccess(clazz);
        }
    }
    
    public static void checkPackageAccess(String s) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            String s2 = s.replace('/', '.');
            if (s2.startsWith("[")) {
                int beginIndex = s2.lastIndexOf(91) + 2;
                if (beginIndex > 1 && beginIndex < s2.length()) {
                    s2 = s2.substring(beginIndex);
                }
            }
            int lastIndex = s2.lastIndexOf(46);
            if (lastIndex != -1) {
                securityManager.checkPackageAccess(s2.substring(0, lastIndex));
            }
        }
    }
    
    public static boolean isPackageAccessible(Class<?> clazz) {
        try {
            checkPackageAccess(clazz);
        }
        catch (SecurityException ex) {
            return false;
        }
        return true;
    }
    
    private static boolean isAncestor(ClassLoader classLoader, ClassLoader parent) {
        do {
            parent = parent.getParent();
            if (classLoader == parent) {
                return true;
            }
            continue;
        } while (parent != null);
        return false;
    }
    
    public static boolean needsPackageAccessCheck(ClassLoader classLoader, ClassLoader classLoader2) {
        return classLoader != null && classLoader != classLoader2 && (classLoader2 == null || !isAncestor(classLoader, classLoader2));
    }
    
    public static void checkProxyPackageAccess(Class<?> cl) {
        if (System.getSecurityManager() != null && Proxy.isProxyClass(cl)) {
            Class[] interfaces = cl.getInterfaces();
            for (int length = interfaces.length, i = 0; i < length; ++i) {
                checkPackageAccess(interfaces[i]);
            }
        }
    }
    
    public static void checkProxyPackageAccess(ClassLoader classLoader, Class<?>... array) {
        if (System.getSecurityManager() != null) {
            for (Class<?> clazz : array) {
                if (needsPackageAccessCheck(classLoader, clazz.getClassLoader())) {
                    checkPackageAccess(clazz);
                }
            }
        }
    }
    
    public static boolean isNonPublicProxyClass(Class<?> cl) {
        String name = cl.getName();
        int lastIndex = name.lastIndexOf(46);
        String s = (lastIndex != -1) ? name.substring(0, lastIndex) : "";
        return Proxy.isProxyClass(cl) && !s.equals("com.sun.proxy");
    }
    
    public static void checkProxyMethod(Object o, Method obj) {
        if (o == null || !Proxy.isProxyClass(o.getClass())) {
            throw new IllegalArgumentException("Not a Proxy instance");
        }
        if (Modifier.isStatic(obj.getModifiers())) {
            throw new IllegalArgumentException("Can't handle static method");
        }
        Class<Object> declaringClass = obj.getDeclaringClass();
        if (declaringClass == Object.class) {
            String name = obj.getName();
            if (name.equals("hashCode") || name.equals("equals") || name.equals("toString")) {
                return;
            }
        }
        if (isSuperInterface(o.getClass(), declaringClass)) {
            return;
        }
        throw new IllegalArgumentException("Can't handle: " + obj);
    }
    
    private static boolean isSuperInterface(Class<?> clazz, Class<?> clazz2) {
        Class[] interfaces = clazz.getInterfaces();
        int length = interfaces.length;
        int i = 0;
        while (i < length) {
            Class<Object> clazz3 = interfaces[i];
            if (clazz3 == clazz2) {
                return true;
            }
            if (isSuperInterface(clazz3, clazz2)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static boolean isVMAnonymousClass(Class<?> clazz) {
        return clazz.getName().indexOf("/") > -1;
    }
}
